#pragma once

#ifdef LINUX
#include "../../include/AnalysisModuleInterface.h"
#include "../../include/InOutPin.h"
#endif

#define WINDOWS

#include <stdio.h>
//#include <math.h>	// for atof
#include <string>

//#include "LICAnalyzer.h"

#ifndef	WINDOWS
#include <unistd.h>
#include <stdlib.h>
#endif

#include "LVariableItem.h"
#include "LVarContainer.h"
#include <ctype.h>

#define JUDGEMENT_PERCENTAGE		(0.75)
#define FILE_READ_SIZE				512

#define MAX_INT_VALUE				(2147483647*JUDGEMENT_PERCENTAGE)
#define MIN_INT_VALUE				((-2147483647)*JUDGEMENT_PERCENTAGE)
#define MAX_UNSIGNED_INT_VALUE		((unsigned int)(4294967295/100*JUDGEMENT_PERCENTAGE*100))
#define MIN_UNSIGNED_INT_VALUE		0
#define MAX_CHAR_VALUE				(127*JUDGEMENT_PERCENTAGE)			// signed char in linux
#define MIN_CHAR_VALUE				((-127)*JUDGEMENT_PERCENTAGE)
#define MAX_UNSIGNED_CHAR_VALUE		(255*JUDGEMENT_PERCENTAGE)
#define MIN_UNSIGNED_CHAR_VALUE		0
#define MAX_SHORT_VALUE				(32768*JUDGEMENT_PERCENTAGE)
#define MIN_SHORT_VALUE				((-32768)*JUDGEMENT_PERCENTAGE)
#define MAX_UNSIGNED_SHORT_VALUE	(65535*JUDGEMENT_PERCENTAGE)
#define MIN_UNSIGNED_SHORT_VALUE	0

#define MAX_LONG_VALUE				MAX_INT_VALUE
#define MIN_LONG_VALUE				MIN_INT_VALUE
#define MAX_UNSIGNED_LONG_VALUE		MAX_UNSIGNED_INT_VALUE
#define MIN_UNSIGNED_LONG_VALUE		0

#define MAX_FLOAT_VALUE				((3.4E+38)*JUDGEMENT_PERCENTAGE)
#define MIN_FLOAT_VALUE				((-3.4E+38)*JUDGEMENT_PERCENTAGE)
#define MAX_DOUBLE_VALUE			((1.7E+308)*JUDGEMENT_PERCENTAGE)
#define MIN_DOUBLE_VALUE			((-1.7E+308)*JUDGEMENT_PERCENTAGE)

#define IC_ERROR_PARSING		1
#define IC_ERROR_FILEREAD	2
#define IC_ERROR_FILEOPEN	3
#define IC_ERROR_VARNAME		4

//Incorrect Calculation

class LICAnalyzer
{
public:
	LVarContainer	m_VarManager;
	stack<string>	m_stackVarValue;
	stack<char>		m_stackOperator;
	stack<char>		m_stackBrace;

	FILE*			m_pFile;
	int			m_nFileSize;
	string			m_strFileName;
	char			m_bufRead[ FILE_READ_SIZE + 1 ];
	char			m_bufTemp[ FILE_READ_SIZE + 1 ];

	int			m_nRead;
	int			m_nLine;
	int			m_nRate;

	bool			m_bFirstWord;

	static CAnalysisNotifyInterface* m_pInterface;
	static bool m_bStopAnalyzer;

public:
	LICAnalyzer();
	~LICAnalyzer();

	void RightTrim(string& strSrc);
	void LeftTrim(string& strSrc);

	int Priority(char cOperator);

	long long Calculate(string strFormula, int nType);
	string Operate(string& strValue1, string& strValue2, char cOperator);
	bool DeclareVar(string& strLine, int nType);
	int ReadFile(string& strBuffer);
	void AnalyzeFile(char* pFileName);
	void AnalyzeFunction(string& strFunction);
	bool ScanVarDeclare(string& strLine);

	void NotifyResult();
	void NotifyError(int nErrorNum, char* pErrorText);
	void NotifyProcess();

	static void SetNotifyInterface(CAnalysisNotifyInterface* pInterface);
	static void SetStopValue(bool bStop);
	

};

LICAnalyzer::LICAnalyzer()
{
	SetStopValue( true );
	memset( m_bufRead, 0x00, FILE_READ_SIZE + 1 );
	memset( m_bufTemp, 0x00, FILE_READ_SIZE + 1 );
}
LICAnalyzer::~LICAnalyzer()
{
}

void LICAnalyzer::RightTrim(string& strSrc)
{
	int nIndex = strSrc.find_last_not_of(" \t\n\r");
	if( nIndex != string::npos )
		strSrc.erase( nIndex + 1 );
}

void LICAnalyzer::LeftTrim(string& strSrc)
{
	int nIndex = strSrc.find_first_not_of(" \t\n\r");
	if( nIndex != string::npos )
		strSrc.erase( 0, nIndex );
}

int LICAnalyzer::Priority(char cOperator)
{
	switch( cOperator )
	{
	case '(':
	case ')':
		return 20;
	case '*':
	case '/':
	case '%':
		return 19;
	case '+':
	case '-':
		return 18;
	default:
		return 0;
	}
}


long long LICAnalyzer::Calculate(string strFormula, int nType)
{
	int nSpaceIndex;
	while( string::npos != (nSpaceIndex = strFormula.find_first_of(" \t\n\r")) )
		strFormula.erase( nSpaceIndex, 1 );

	LVariableItem *pVarItem;
	string strOperator = "+-*%/();";
	string strWord;
	int nOperatorIndex;
	char cLetter;
	char cOperator;
	
	strFormula += ";";	// 파싱을 위해 임의로 집어넣음

	for( int nPos = 0; nPos < strFormula.length(); nPos++ )
	{
		cLetter = strFormula[ nPos ];

		if( isalpha(cLetter) || cLetter == '_' )	// 변수 이름인 경우 변수값을 읽어서 스택에 저장한다.
		{
			if( string::npos != (nOperatorIndex = strFormula.find_first_of(strOperator, nPos+1)) )
			{
				strWord = strFormula.substr( nPos, nOperatorIndex - nPos );
				if( NULL != (pVarItem = m_VarManager.FindVarItem(strWord)) )
				{
					pVarItem->ValueToStr( strWord );
					m_stackVarValue.push( strWord );	// 변수는 값을 스트링으로 넣는다.
					nPos = nOperatorIndex;
				}
				else
				{
					NotifyError( IC_ERROR_VARNAME, "Error : Undeclared Variable"); // 선언되지 않은 변수 접근
					return 0;
				}
			}
			else
			{
				NotifyError( IC_ERROR_PARSING, "Error : Can't Analyze");	// ( 파싱오류. 소스코드 이상 의심 )
				return 0;
			}
		}
		else if( isdigit(cLetter) )
		{
			if( string::npos != (nOperatorIndex = strFormula.find_first_of(strOperator, nPos+1)) )
			{
				strWord = strFormula.substr( nPos, nOperatorIndex - nPos );
				m_stackVarValue.push( strWord );	// 숫자는 그냥 넣는다.
				nPos = nOperatorIndex;
			}
			else
			{
				NotifyError( IC_ERROR_PARSING, "Error : Can't Analyze");	// ( 파싱오류. 소스코드 이상 의심 )
				return 0;
			}
		}
		else if( cLetter == '-' )
		{
			if( nPos == 0 || (isalpha(strFormula[ nPos - 1 ]) || isdigit(strFormula[ nPos - 1 ])) )
			{
				nPos++;
				cLetter = strFormula[ nPos ];

				if( isalpha(cLetter) || cLetter == '_' )	// 뒤에 나오는 변수값의 부호를 바꿔서 스택에 저장
				{
					if( string::npos != (nOperatorIndex = strFormula.find_first_of(strOperator, nPos+1)) )
					{
						strWord = strFormula.substr( nPos, nOperatorIndex - nPos );
						if( NULL != (pVarItem = m_VarManager.FindVarItem(strWord)) )
						{
							pVarItem->ValueToStr( strWord );
							if( strWord[ 0 ] == '-' )	// 부호를 바꿈
								strWord.erase( 0, 1 );
							else
								strWord = ("-" + strWord);
							m_stackVarValue.push( strWord );
							nPos = nOperatorIndex;
						}
						else
							NotifyError( IC_ERROR_VARNAME, "Error : Undeclared Variable" ); // 선언되지 않은 변수 접근
					}
					else
					{
						NotifyError( IC_ERROR_PARSING, "Error : Can't Analyze" );	// ( 파싱오류. 소스코드 이상 의심 )
						return 0;
					}
				}
				else if( isdigit(cLetter) )	// 뒤에 나오는 숫자의 부호를 바꿔서 스택에 저장
				{
					if( string::npos != (nOperatorIndex = strFormula.find_first_of(strOperator, nPos+1)) )
					{
						strWord = strFormula.substr( nPos, nOperatorIndex - nPos );
						strWord = ("-" + strWord);
						m_stackVarValue.push( strWord );
						nPos = nOperatorIndex;
					}
					else
					{
						NotifyError( IC_ERROR_PARSING, "Error : Can't Analyze" );	// ( 파싱오류. 소스코드 이상 의심 )
						return 0;
					}
				}
				else
				{
					NotifyError( IC_ERROR_PARSING, "Error : Can't Analyze" );	// ( 파싱오류. 소스코드 이상 의심 )
					return 0;
				}
			}
			else	// 그냥 빼기 연산자인 경우
			{
				m_stackOperator.push( '-' );
			}
		}// end == '-'
		else if( cLetter == ')' )	// ( 가 나올때 까지 팝하여 계산하여 다시 푸쉬
		{
			while( m_stackOperator.top() != '(' )
			{
				cLetter = m_stackOperator.top();
				m_stackOperator.pop();
				strWord = m_stackVarValue.top();
				m_stackVarValue.pop();
				strWord = Operate( m_stackVarValue.top(), strWord, cLetter );
				m_stackVarValue.pop();
				m_stackVarValue.push( strWord );
			}
			m_stackOperator.pop();
		}
		else	// 그외에 operator 인 경우
		{
			if( m_stackOperator.empty() )
				m_stackOperator.push( cLetter );
			else
			{
				if( Priority( m_stackOperator.top() ) < Priority( cLetter ) )
				{
					// 스택에서 숫자 하나 꺼내고, 지금 연산자 다음 숫자를 이번연산자로 계산하여 스택에 푸쉬
					if( string::npos != (nOperatorIndex = strFormula.find_first_of(strOperator, nPos+1)) )
					{
						strWord = strFormula.substr( nPos + 1, nOperatorIndex - nPos - 1 );
						LeftTrim( strWord );
						RightTrim( strWord );
						strWord = Operate( m_stackVarValue.top(), strWord, cLetter );
						m_stackVarValue.pop();
						m_stackVarValue.push( strWord );
						nPos = nOperatorIndex;
					}
					strWord = m_stackVarValue.top();
					m_stackVarValue.pop();
					
				}
				else
					m_stackOperator.push( cLetter );
			}
		}

	}// end for( nPos++ )
	
	// 숫자와 연산자를 스택에서 꺼내어 계산시작
	while( m_stackOperator.empty() == false )
	{
		cLetter = m_stackOperator.top();
		m_stackOperator.pop();
		strWord = m_stackVarValue.top();
		m_stackVarValue.pop();
		strWord = Operate( m_stackVarValue.top(), strWord, cLetter );
		m_stackVarValue.pop();
		m_stackVarValue.push( strWord );
	}

	strWord = m_stackVarValue.top();
	m_stackVarValue.pop();

	// 마지막 결과값과 변수형의 최대 표현 범위와 비교	

	if( string::npos != strWord.find(".") )
	{
		double dbResult = atof( strWord.c_str() );
		switch( nType )
		{
			case VAR_TYPE_INT:
				if( dbResult > MAX_INT_VALUE || dbResult < MIN_INT_VALUE )
					NotifyResult();
				break;
			case VAR_TYPE_UNSIGNED_INT:
				if( dbResult > MAX_UNSIGNED_INT_VALUE || dbResult < MIN_UNSIGNED_INT_VALUE )
					NotifyResult();
				break;
			case VAR_TYPE_CHAR:
				if( dbResult > MAX_CHAR_VALUE || dbResult < MIN_CHAR_VALUE )
					NotifyResult();
				break;
			case VAR_TYPE_UNSIGNED_CHAR:
				if( dbResult > MAX_UNSIGNED_CHAR_VALUE || dbResult < MIN_UNSIGNED_CHAR_VALUE )
					NotifyResult();
				break;
			case VAR_TYPE_SHORT:
				if( dbResult > MAX_SHORT_VALUE || dbResult < MIN_SHORT_VALUE )
					NotifyResult();
				break;
			case VAR_TYPE_UNSIGNED_SHORT:
				if( dbResult > MAX_UNSIGNED_SHORT_VALUE || dbResult < MIN_UNSIGNED_SHORT_VALUE )
					NotifyResult();
				break;
			case VAR_TYPE_LONG:
				if( dbResult > MAX_LONG_VALUE || dbResult < MIN_LONG_VALUE )
					NotifyResult();
				break;
			case VAR_TYPE_UNSIGNED_LONG:
				if( dbResult > MAX_UNSIGNED_LONG_VALUE || dbResult < MIN_UNSIGNED_LONG_VALUE )
					NotifyResult();
				break;
			//case VAR_TYPE_LONG_LONG:
			//case VAR_TYPE_UNSIGNED_LONG_LONG:
			case VAR_TYPE_FLOAT:
				if( dbResult > MAX_FLOAT_VALUE || dbResult < MIN_FLOAT_VALUE )
					NotifyResult();
				break;
			case VAR_TYPE_DOUBLE:
				if( dbResult > MAX_DOUBLE_VALUE || dbResult < MIN_DOUBLE_VALUE )
					NotifyResult();
				break;
			default:
				break;
		}
		long long llResult = 0;
		memcpy( &llResult, &dbResult, sizeof(dbResult) );
		return llResult;
	}
	else
	{
		int nResult = atol( strWord.c_str() );
		switch( nType )
		{
			case VAR_TYPE_INT:
				if( nResult > MAX_INT_VALUE || nResult < MIN_INT_VALUE )
					NotifyResult();
				break;
			case VAR_TYPE_UNSIGNED_INT:
				if( nResult > MAX_UNSIGNED_INT_VALUE || nResult < MIN_UNSIGNED_INT_VALUE )
					NotifyResult();
				break;
			case VAR_TYPE_CHAR:
				if( nResult > MAX_CHAR_VALUE || nResult < MIN_CHAR_VALUE )
					NotifyResult();
				break;
			case VAR_TYPE_UNSIGNED_CHAR:
				if( nResult > MAX_UNSIGNED_CHAR_VALUE || nResult < MIN_UNSIGNED_CHAR_VALUE )
					NotifyResult();
				break;
			case VAR_TYPE_SHORT:
				if( nResult > MAX_SHORT_VALUE || nResult < MIN_SHORT_VALUE )
					NotifyResult();
				break;
			case VAR_TYPE_UNSIGNED_SHORT:
				if( nResult > MAX_UNSIGNED_SHORT_VALUE || nResult < MIN_UNSIGNED_SHORT_VALUE )
					NotifyResult();
				break;
			case VAR_TYPE_LONG:
				if( nResult > MAX_LONG_VALUE || nResult < MIN_LONG_VALUE )
					NotifyResult();
				break;
			case VAR_TYPE_UNSIGNED_LONG:
				if( nResult > MAX_UNSIGNED_LONG_VALUE || nResult < MIN_UNSIGNED_LONG_VALUE )
					NotifyResult();
				break;
			//case VAR_TYPE_LONG_LONG:
			//case VAR_TYPE_UNSIGNED_LONG_LONG:
			case VAR_TYPE_FLOAT:
				if( nResult > MAX_FLOAT_VALUE || nResult < MIN_FLOAT_VALUE )
					NotifyResult();
				break;
			case VAR_TYPE_DOUBLE:
				if( nResult > MAX_DOUBLE_VALUE || nResult < MIN_DOUBLE_VALUE )
					NotifyResult();
				break;
			default:
				break;
		}
		return nResult;
	}
}

string LICAnalyzer::Operate(string& strValue1, string& strValue2, char cOperator)
{
	bool bIsDouble1 = false, bIsDouble2 = false;
	long long llValue1=0, llValue2=0;
	double	dbValue1=0, dbValue2=0;

	if( string::npos != strValue1.find(".") )
	{
		dbValue1 = atof( strValue1.c_str() );
		bIsDouble1 = true;
	}
	else
	{
		llValue1 = atol( strValue1.c_str() );
	}

	if( string::npos != strValue2.find(".") )
	{
		dbValue2 = atof( strValue2.c_str() );
		bIsDouble2 = true;
	}
	else
	{
		llValue2 = atol( strValue2.c_str() );
	}

	char buf[30];
	memset( buf, 0x00, 30 );
	string strResult;

	if( bIsDouble1 || bIsDouble2 )
	{
		dbValue1 = llValue1 + dbValue1;
		dbValue2 = llValue2 + dbValue2;

		switch( cOperator )
		{
		case '+':
			strResult = gcvt( dbValue1 + dbValue2, 20, buf );
			break;
		case '-':
			strResult = gcvt( dbValue1 - dbValue2, 20, buf );
			break;
		case '*':
			strResult = gcvt( dbValue1 * dbValue2, 20, buf );
			break;
		case '/':
			strResult = gcvt( dbValue1 / dbValue2, 20, buf );
			break;
		case '%':
			strResult = gcvt( (long)dbValue1 % (long)dbValue2, 20, buf );
			break;
		default:
			break;
		}
	}
	else
	{
		switch( cOperator )
		{
		case '+':
			strResult = itoa( llValue1 + llValue2, buf, 10 );
			break;
		case '-':
			strResult = itoa( llValue1 - llValue2, buf, 10 );
			break;
		case '*':
			strResult = itoa( llValue1 * llValue2, buf, 10 );
			break;
		case '/':
			strResult = itoa( llValue1 / llValue2, buf, 10 );
			break;
		case '%':
			strResult = itoa( llValue1 % llValue2, buf, 10 );
			break;
		default:
			break;
		}
	}	

	return strResult;	
}

bool LICAnalyzer::DeclareVar(string& strLine, int nType)
{
	int nBeginIndex;
	int nEndIndex;
	int nTokenIndex;
	string strDeclareToken = ",;";
	string strOne;
	string strVar;
	long long llValue = 0;
	bool bIsDeclaration = false;

	// 왼쪽 공백 삭제
	LeftTrim( strLine );
	nBeginIndex = 0;

	while( string::npos != (nEndIndex = strLine.find_first_of(strDeclareToken)) )
	{
		strOne = strLine.substr( 0, nEndIndex );	// 변수 선언 하나를 뽑아냄. 토큰은 제외

		if( string::npos != (nTokenIndex = strOne.find( "=" )) )
		{
			strVar = strOne.substr( 0, nTokenIndex );	// 변수 이름
			LeftTrim( strVar );
			RightTrim( strVar );
			strOne.erase( 0, nTokenIndex + 1 );	// 변수에 들어갈 식만 남기고 =까지 삭제
			llValue |= Calculate( strOne, nType );	// 계산. ★ 여기서 위험성 검사 ; 제외되어 전달
			m_VarManager.PushVarItem( strVar, nType, llValue );
		}
		else
		{
			LeftTrim( strOne );
			strVar = strOne;

			m_VarManager.PushVarItem( strVar, nType );
		}

		strLine = strLine.erase( 0, nEndIndex + 1 );
		
		LeftTrim( strLine );

		bIsDeclaration = true;
	}

	return bIsDeclaration;

}

bool LICAnalyzer::ScanVarDeclare(string& strLine)
{
	// ;을 포함한 한줄을 처리한다.
	int nIndex;
	
	if( string::npos != (nIndex = strLine.find( "unsigned int" )) )
	{
		nIndex = nIndex + strlen("unsigned int");
		if( isspace(strLine[ nIndex ]) )	// 뒤에 공백이 하나 있으면
		{
			strLine = strLine.substr( nIndex + 1 );	// 공백이후글자부터 끝까지
			return DeclareVar( strLine, VAR_TYPE_UNSIGNED_INT );	
		}
	}
	if( string::npos != (nIndex = strLine.find( "signed int" )) )
	{
		nIndex = nIndex + strlen("signed int");
		if( isspace(strLine[ nIndex ]) )	// 뒤에 공백이 하나 있으면
		{
			strLine = strLine.substr( nIndex + 1 );	// 공백이후글자부터 끝까지
			return DeclareVar( strLine, VAR_TYPE_SIGNED_INT );
		}
	}	
	if( string::npos != (nIndex = strLine.find( "signed char" )) )
	{
		nIndex = nIndex + strlen("signed char");
		if( isspace(strLine[ nIndex ]) )	// 뒤에 공백이 하나 있으면
		{
			strLine = strLine.substr( nIndex + 1 );	// 공백이후글자부터 끝까지
			return DeclareVar( strLine, VAR_TYPE_SIGNED_CHAR );			
		}
	}
	if( string::npos != (nIndex = strLine.find( "short int" )) )
	{
		nIndex = nIndex + strlen("short int");
		if( isspace(strLine[ nIndex ]) )	// 뒤에 공백이 하나 있으면
		{
			strLine = strLine.substr( nIndex + 1 );	// 공백이후글자부터 끝까지
			return DeclareVar( strLine, VAR_TYPE_SHORT_INT );
		}
	}
	if( string::npos != (nIndex = strLine.find( "unsigned short" )) )
	{
		nIndex = nIndex + strlen("unsigned short");
		if( isspace(strLine[ nIndex ]) )	// 뒤에 공백이 하나 있으면
		{
			strLine = strLine.substr( nIndex + 1 );	// 공백이후글자부터 끝까지
			return DeclareVar( strLine, VAR_TYPE_UNSIGNED_SHORT );	
		}
	}
	if( string::npos != (nIndex = strLine.find( "signed short" )) )
	{
		nIndex = nIndex + strlen("signed short");
		if( isspace(strLine[ nIndex ]) )	// 뒤에 공백이 하나 있으면
		{
			strLine = strLine.substr( nIndex + 1 );	// 공백이후글자부터 끝까지
			return DeclareVar( strLine, VAR_TYPE_SIGNED_SHORT );
		}
	}
	if( string::npos != (nIndex = strLine.find( "long int" )) )
	{
		nIndex = nIndex + strlen("long int");
		if( isspace(strLine[ nIndex ]) )	// 뒤에 공백이 하나 있으면
		{
			strLine = strLine.substr( nIndex + 1 );	// 공백이후글자부터 끝까지
			return DeclareVar( strLine, VAR_TYPE_LONG_INT );	
		}
	}
	if( string::npos != (nIndex = strLine.find( "unsigned long" )) )
	{
		nIndex = nIndex + strlen("unsigned long");
		if( isspace(strLine[ nIndex ]) )	// 뒤에 공백이 하나 있으면
		{
			strLine = strLine.substr( nIndex + 1 );	// 공백이후글자부터 끝까지
			return DeclareVar( strLine, VAR_TYPE_UNSIGNED_LONG );	
		}
	}
	if( string::npos != (nIndex = strLine.find( "signed long" )) )
	{
		nIndex = nIndex + strlen("signed long");
		if( isspace(strLine[ nIndex ]) )	// 뒤에 공백이 하나 있으면
		{
			strLine = strLine.substr( nIndex + 1 );	// 공백이후글자부터 끝까지
			return DeclareVar( strLine, VAR_TYPE_SIGNED_LONG );
		}
	}
	//if( string::npos != (nIndex = strLine.find( "long long" )) )
	//{
	//	nIndex = nIndex + strlen("long long");
	//	if( isspace(strLine[ nIndex ]) )	// 뒤에 공백이 하나 있으면
	//	{
	//		strLine = strLine.substr( nIndex + 1 );	// 공백이후글자부터 끝까지
	//		return DeclareVar( strLine, VAR_TYPE_LONG_LONG );		
	//	}
	//}
	//if( string::npos != (nIndex = strLine.find( "unsigned long long" )) )
	//{
	//	nIndex = nIndex + strlen("unsigned long long");
	//	if( isspace(strLine[ nIndex ]) )	// 뒤에 공백이 하나 있으면
	//	{
	//		strLine = strLine.substr( nIndex + 1 );	// 공백이후글자부터 끝까지
	//		return DeclareVar( strLine, VAR_TYPE_UNSIGNED_LONG_LONG );			
	//	}
	//}
	if( string::npos != (nIndex = strLine.find( "long double" )) )
	{
		nIndex = nIndex + strlen("long double");
		if( isspace(strLine[ nIndex ]) )	// 뒤에 공백이 하나 있으면
		{
			strLine = strLine.substr( nIndex + 1 );	// 공백이후글자부터 끝까지
			return DeclareVar( strLine, VAR_TYPE_LONG_DOUBLE );			
		}
	}
	if( string::npos != (nIndex = strLine.find( "float" )) )
	{
		nIndex = nIndex + strlen("float");
		if( isspace(strLine[ nIndex ]) )	// 뒤에 공백이 하나 있으면
		{
			strLine = strLine.substr( nIndex + 1 );	// 공백이후글자부터 끝까지
			return DeclareVar( strLine, VAR_TYPE_FLOAT );			
		}
	}
	if( string::npos != (nIndex = strLine.find( "double" )) )
	{
		nIndex = nIndex + strlen("double");
		if( isspace(strLine[ nIndex ]) )	// 뒤에 공백이 하나 있으면
		{
			strLine = strLine.substr( nIndex + 1 );	// 공백이후글자부터 끝까지
			return DeclareVar( strLine, VAR_TYPE_DOUBLE );			
		}
	}
	if( string::npos != (nIndex = strLine.find( "int" )) )
	{
		nIndex = nIndex + strlen("int");
		if( isspace(strLine[ nIndex ]) )	// 뒤에 공백이 하나 있으면
		{
			strLine = strLine.substr( nIndex + 1 );	// 공백이후글자부터 끝까지
			return DeclareVar( strLine, VAR_TYPE_INT );			
		}
	}
	if( string::npos != (nIndex = strLine.find( "char" )) )
	{
		nIndex = nIndex + strlen("char");
		if( isspace(strLine[ nIndex ]) )	// 뒤에 공백이 하나 있으면
		{
			strLine = strLine.substr( nIndex + 1 );	// 공백이후글자부터 끝까지
			return DeclareVar( strLine, VAR_TYPE_CHAR );			
		}
	}
	if( string::npos != (nIndex = strLine.find( "byte" )) )
	{
		nIndex = nIndex + strlen("byte");
		if( isspace(strLine[ nIndex ]) )	// 뒤에 공백이 하나 있으면
		{
			strLine = strLine.substr( nIndex + 1 );	// 공백이후글자부터 끝까지
			return DeclareVar( strLine, VAR_TYPE_BYTE );			
		}
	}
	if( string::npos != (nIndex = strLine.find( "short" )) )
	{
		nIndex = nIndex + strlen("short");
		if( isspace(strLine[ nIndex ]) )	// 뒤에 공백이 하나 있으면
		{
			strLine = strLine.substr( nIndex + 1 );	// 공백이후글자부터 끝까지
			return DeclareVar( strLine, VAR_TYPE_SHORT );			
		}
	}
	if( string::npos != (nIndex = strLine.find( "long" )) )
	{
		nIndex = nIndex + strlen("long");
		if( isspace(strLine[ nIndex ]) )	// 뒤에 공백이 하나 있으면
		{
			strLine = strLine.substr( nIndex + 1 );	// 공백이후글자부터 끝까지
			return DeclareVar( strLine, VAR_TYPE_LONG );			
		}
	}

	return false;
}

int LICAnalyzer::ReadFile(string& strBuffer)
{
	memset( m_bufRead, 0x00, FILE_READ_SIZE + 1 );
	int nRead = fread( m_bufRead, 1, FILE_READ_SIZE, m_pFile );

	if( nRead < 0 )
	{
		NotifyError( IC_ERROR_FILEREAD, "Error : File Read" );
		return -1;
	}

	m_nRead += nRead;
	strBuffer += m_bufRead;
	return nRead;
}

void LICAnalyzer::AnalyzeFile(char* pFileName)
{
	m_pFile = fopen( pFileName, "r" );
	if( m_pFile == NULL )
	{
		NotifyError( IC_ERROR_FILEOPEN, "Error : File Open" );
		return;
	}
	fseek( m_pFile, 0, SEEK_END );
	m_nFileSize = ftell( m_pFile );
	fseek( m_pFile, 0, SEEK_SET );

	m_strFileName = pFileName;

	string strBuffer = "";
	m_nLine = 1;
	m_nRate = 0;
	m_nRead = 0;

	if( ReadFile( strBuffer ) == false )
		return;

	while( true )	// 파일하나를 다 분석할때 까지
	{
		if( m_bStopAnalyzer == true )
			break;

		int nBeginningIndex = 0;
		int nOpenBraceIndex = strBuffer.find( "{" );	// { 를 검색

		if( nOpenBraceIndex == string::npos )	// { 가 없는 경우
		{
			// 줄번호 계산
			while( string::npos != (nBeginningIndex = strBuffer.find("\n", nBeginningIndex)) )
			{
				nBeginningIndex++;
				m_nLine++;				
			}
			strBuffer = "";

			if( ReadFile( strBuffer ) == false )	// { 가 없으면 read
				break;

			continue;
		}

		// 줄번호 계산
		while( string::npos != (nBeginningIndex = strBuffer.find("\n", nBeginningIndex)) )
		{
			if( nBeginningIndex > nOpenBraceIndex )
				break;

			nBeginningIndex++;
			m_nLine++;
		}

		m_stackBrace.push( '{' );

		int nBraceIndex = nBeginningIndex + 1;
		int nCloseBraceIndex;

		while( m_stackBrace.empty() == false )
		{
			if( string::npos == (nBraceIndex = strBuffer.find_first_of("{}", nBraceIndex)) )
			{
				if( ReadFile( strBuffer ) == false )
				{
					SetStopValue( true );
					break;
				}	
			}
			else if( strBuffer[ nBraceIndex ] == '{' )
			{
				m_stackBrace.push( '{' );
				nBraceIndex++;
			}
			else
			{
				m_stackBrace.pop();
				nCloseBraceIndex = nBraceIndex;
				nBraceIndex++;
			}
		}

		if( m_bStopAnalyzer == true )
			break;

		// { + 1 부터 } - 1 까지 스트링만 서브스트링으로 잘라서 검사
		string temp = strBuffer.substr(nOpenBraceIndex + 1, nCloseBraceIndex - nOpenBraceIndex - 2);
		AnalyzeFunction( temp );

		if( m_bStopAnalyzer == true )
			break;

		strBuffer.erase( 0, nCloseBraceIndex + 1 );

		m_nRate = m_nRead / m_nFileSize * 100;

	}// end while
	

	// TODO here : 변수 관리 스택에 내용이 남아있으면 } 이 나오지 않은 것이므로 오류 출력

	fclose( m_pFile );
	
}

void LICAnalyzer::AnalyzeFunction(string& strFunction)
{
	int nBeginningIndex = 0;
	int nSemicolonIndex;

	// 변수선언 부분 검사
	while( string::npos != (nSemicolonIndex = strFunction.find(";")) )
	{
		if( m_bStopAnalyzer == true )
			return;

		// 줄번호 계산
		while( string::npos != (nBeginningIndex = strFunction.find("\n", nBeginningIndex)) )
		{
			if( nBeginningIndex > nSemicolonIndex )
				break;

			nBeginningIndex++;
			m_nLine++;
		}

		// 한줄에 대해 변수 선언부분인지 검사( ;까지 전달)
		string temp = strFunction.substr( 0, nSemicolonIndex + 1);
		if( true == ScanVarDeclare( temp ) )
		{
			strFunction.erase( 0, nSemicolonIndex + 1 );
			nBeginningIndex = 0;	// 줄번호 계산을 위함
		}
		else
			break;

	}

	int nOperatorIndex;
	int nEndLineIndex;	// = 글자가 나오기 바로 이전 \n의 인덱스
	string strVar;
	long long llValue;
	int i;

	// 변수 값 검사
	//while( string::npos != (nOperatorIndex = strFunction.find_first_of("=+-")) )
	while( string::npos != (nOperatorIndex = strFunction.find_first_of("=")) )
	{
		if( m_bStopAnalyzer == true )
			return;

		nEndLineIndex = 0;

		// 줄번호 계산
		while( string::npos != (nBeginningIndex = strFunction.find("\n", nBeginningIndex)) )
		{
			if( nBeginningIndex > nOperatorIndex )
				break;

			nEndLineIndex = nBeginningIndex;
			nBeginningIndex++;
			m_nLine++;
		}

		if( strFunction[ nOperatorIndex ] == '=' && strFunction[ nOperatorIndex + 1 ] != '=' )	// == 이 아닌지 확인
		{
			for( i = nEndLineIndex + 1; i < nOperatorIndex; i++ )
			{
				if( isalpha(strFunction[ i ]) || strFunction[ i ] == '_' )	// 변수이름인지 확인
				{
					if( string::npos != (nSemicolonIndex = strFunction.find( ";", nOperatorIndex + 2)) )
					{
						strVar = strFunction.substr( i, nOperatorIndex - i );
						RightTrim( strVar );
						// 변수 이름이 관리스택에 있으면 Calculate()
						LVariableItem* pVarItem = m_VarManager.FindVarItem( strVar );
						if( NULL != pVarItem )
						{
							// 세미콜론 제외하고 전달
							llValue = Calculate( strFunction.substr(nOperatorIndex+1, nSemicolonIndex - nOperatorIndex-1), pVarItem->GetType() );
							m_VarManager.SetVarItem( strVar, llValue );
						}

						break;
					}
					else
					{
						return;	// TODO here : error report : =은 있는데 ;이 없음
					}
				}
			}
			if( i == nOperatorIndex )
				return;	// TODO here : error report : 이퀄 앞에 변수 이름 없음

			strFunction.erase( 0, nSemicolonIndex + 1 );	// ; 까지 삭제
		}
		else
			strFunction.erase( 0, nOperatorIndex + 2 );	// == 까지 삭제

		// TODO here
		/*
		else if( strFunction[ nOperatorIndex ] == '+' && strFunction[ nOperatorIndex + 1 ] == '+' )	// ++ 이 아닌지 확인
		{
		}
		else if( strFunction[ nOperatorIndex ] == '-' && strFunction[ nOperatorIndex + 1 ] == '-' )	// -- 이 아닌지 확인
		{
		}*/
	}
}

void LICAnalyzer::SetNotifyInterface(CAnalysisNotifyInterface* pInterface)
{
	m_pInterface = pInterface;
}

void LICAnalyzer::SetStopValue(bool bStop)
{
	m_bStopAnalyzer = bStop;
}

void LICAnalyzer::NotifyResult()
{	
	char *pName = new char[ m_strFileName.length() + 1 ];
	memset( pName, 0x00, m_strFileName.length() + 1 );
	memcpy( pName, m_strFileName.c_str(), m_strFileName.length() );

	CAnalysisResultPin result;
	result.m_pszFileName = pName;
	result.m_nLineNo = m_nLine;
	result.m_pszErrorText = "Incorrect Calculation can be occurred";
	m_pInterface->OnNotifyAnalysisResult ( MODULE_ID , &result );
	delete [] pName;
}

void LICAnalyzer::NotifyError(int nErrorNum, char* pErrorText)
{
	m_pInterface->OnNotifyError ( MODULE_ID , nErrorNum , pErrorText );
	SetStopValue( true );
}

void LICAnalyzer::NotifyProcess()
{
	CNotifyProcessState state;
	state.m_uiModuleId = MODULE_ID;
	state.m_nProcessRate = m_nRate;
	state.m_pszText = "";
	m_pInterface->OnNotifyProcessState ( &state );
}

